/***********************************************************************
*
* Copyright (c) 2021-2022 Barbara Geller and Ansel Sermersheim
*
* This example is free software, released under the BSD 2-Clause license.
* For license details refer to LICENSE provided with this project.
*
* CopperSpice is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*
* https://opensource.org/licenses/BSD-2-Clause
*
***********************************************************************/

#include <QtCore>
#include <QtGui>

class MainWindow : public QMainWindow
{
 public:
   MainWindow();

 private:
   void selectFile();
   void showAboutJournal();
};

MainWindow::MainWindow()
{
   setMinimumSize(700, 500);

   QWidget *centralWidget = new QWidget(this);
   setCentralWidget(centralWidget);

   // *
   QAction *openFile = new QAction("Open", this);
   openFile->setShortcuts(QKeySequence::Open);
   openFile->setIcon(QIcon("../file_open.png"));
   connect(openFile, &QAction::triggered, this, &MainWindow::selectFile);

   QAction *closeFile = new QAction("Close", this);
   closeFile->setIcon(QIcon("../file_close.png"));

   QAction *saveFile = new QAction("Save", this);
   saveFile->setShortcuts(QKeySequence::Save);
   saveFile->setIcon(QIcon("../file_save.png"));

   QAction *exit = new QAction("Exit", this);
   connect(exit, &QAction::triggered, this, &QWidget::close);

#ifdef Q_OS_WIN
   exit->setShortcut(QKeySequence(Qt::AltModifier | Qt::Key_F4));
#else
   exit->setShortcuts(QKeySequence::Quit);
#endif

   // *
   QAction *undo = new QAction("Undo", this);
   undo->setIcon(QIcon("../undo.png"));
   undo->setStatusTip("Undo the last change");

   QAction *redo = new QAction("Redo", this);
   redo->setIcon(QIcon("../redo.png"));
   redo->setStatusTip("Revert the last undo");

   QAction *cut = new QAction("Cut", this);
   cut->setIcon(QIcon("../cut.png"));
   cut->setToolTip("Tool Tip for Cut");
   cut->setStatusTip("Cut selected text");
   cut->setWhatsThis("Information about Cut (what's this)");

   QAction *copy = new QAction("Copy", this);
   copy->setIcon(QIcon("../copy.png"));
   copy->setToolTip("Tool Tip for Copy");
   copy->setStatusTip("Copy selected text to clipboard");
   copy->setWhatsThis("Information about Copy (what's this)");

   QAction *paste = new QAction("Paste", this);
   paste->setIcon(QIcon("../paste.png"));
   paste->setToolTip("Tool Tip for Paste");
   paste->setStatusTip("Paste contents of clipboard");
   paste->setWhatsThis("Information about Paste (what's this)");

   QAction *about = new QAction("About Journal", this);
   connect(about, &QAction::triggered, this, &MainWindow::showAboutJournal);

   QMenu *fileMenu = menuBar()->addMenu("File");
   fileMenu->addAction(openFile);
   fileMenu->addAction(closeFile);
   fileMenu->addAction(saveFile);
   fileMenu->addSeparator();
   fileMenu->addAction(exit);

   QMenu *editMenu = menuBar()->addMenu("Edit");
   editMenu->addAction(undo);
   editMenu->addAction(redo);
   editMenu->addSeparator();
   editMenu->addAction(cut);
   editMenu->addAction(copy);
   editMenu->addAction(paste);

   QMenu *helpMenu = menuBar()->addMenu("Help");
   helpMenu->addAction(QWhatsThis::createAction());
   helpMenu->addAction(about);

   // *
   QToolBar *fileToolBar = addToolBar("Uncheck to turn off tool bar for File Menu");
   fileToolBar->addAction(openFile);
   fileToolBar->addAction(closeFile);
   fileToolBar->addAction(saveFile);

   QToolBar *editToolBar = addToolBar("Edit Menu");
   editToolBar->addAction(undo);
   editToolBar->addAction(redo);
   editToolBar->addSeparator();
   editToolBar->addAction(cut);
   editToolBar->addAction(copy);
   editToolBar->addAction(paste);

   QToolBar *helpToolBar = addToolBar("Help Menu");
   helpToolBar->addAction(QWhatsThis::createAction());

   // *
   QPushButton *pb_open = new QPushButton();
   pb_open->setText("Open");
   pb_open->setToolTip("<img src='../file_open.png'> Click to Open File");
   pb_open->setWhatsThis("Information about the Open Button (what's this)");

   QPushButton *pb_close = new QPushButton();
   pb_close->setText("Close");

   pb_close->setToolTip("Tool Tip for <b>CLOSE</b> Push Button"
      "<ul>"
      "  <li>Item 1</li>"
      "  <li>Item 2</li>"
      "  <li>Item 3</li>"
      "</ul>");

   pb_close->setWhatsThis("Click button to exit program");

   // *
   statusBar()->showMessage("Temporary Status Message");

   QLabel *statusLine = new QLabel("Permanent Status Message", this);
   statusBar()->addPermanentWidget(statusLine);

   QHBoxLayout *layout = new QHBoxLayout();
   layout->addStretch();
   layout->addWidget(pb_open);
   layout->addSpacing(25);
   layout->addWidget(pb_close);
   layout->addStretch();

   centralWidget->setLayout(layout);

   connect(pb_open,  &QPushButton::clicked, openFile, &QAction::triggered);
   connect(pb_close, &QPushButton::clicked, exit,     &QAction::triggered);
}

void MainWindow::selectFile()
{
   QMessageBox::about(this, "Open File", "User selected the Open File action");
}

void MainWindow::showAboutJournal()
{
   QMessageBox::about(this, "About Journal",
      "Text which typically contains the Developer or Organization Name, Version Number, and Copyright details.");
}

int main(int argc, char *argv[])
{
   QApplication app(argc, argv);

   MainWindow *window = new MainWindow();
   window->show();

   return app.exec();
}
